@@ -31,6 +31,9 @@
 #include <zbar.h>
 #endif
 
+#ifdef N900
+#include <libosso.h>
+#endif
 
 #define DEVICE_UNKNOWN  -1
 #define DEVICE_770  0
@@ -50,14 +53,30 @@
 #define N900_WIDTH  640
 #define N900_HEIGHT  480
 
+#define NO_OF_TABS   4
+
 /* Define structure for variables that
  * are needed thruout the application */
 typedef struct {
     HildonProgram *program;
     HildonWindow *window;
 
+    GtkWidget *tab[NO_OF_TABS];
+    GtkWidget *label[NO_OF_TABS];
+    
+#ifdef FREMANTLE
+    GtkWidget *scan_window_vbox_left;
+    GtkWidget *scan_window_vbox_right;
+    GtkWidget *scan_window_table;
+#endif    
+    
+    
 #ifdef N900
     GstElement *camera_src;
+    DBusConnection *con;
+    osso_context_t *osso;
+    gboolean isportrait;
+    gboolean accelerometers;
 #endif
     GstElement *pipeline;
     GtkWidget *screen;
