@@ -26,6 +26,8 @@
 #include <hildon/hildon-program.h>
 #include <hildon/hildon.h>
 
+#include <libosso.h>
+
 #include <glib.h>
 #include <ctype.h>
 #include <time.h>
@@ -44,6 +46,9 @@
 #include "camera_n8x0.h"
 #endif /* N900 */
 
+#include <mce/dbus-names.h>
+#include <mce/mode-names.h>
+
 
 //gtk_widget_queue_draw(widget);
 
@@ -55,6 +60,143 @@ extern AppData *appdata;
 extern clock_t QR1D_timer;
 extern clock_t dmtx_timer;
 
+// Orientation functions modified from those in conboy.
+
+#define MCE_MATCH_RULE "type='signal',interface='" MCE_SIGNAL_IF "',member='" MCE_DEVICE_ORIENTATION_SIG "'"
+
+static void set_orientation(const gchar* orientation)
+{
+	/* If we disabled the accelerometers, then we don't want to react on it if
+	 * it was turned on by another application.
+	 *
+	 * E.g. when a dialog is open we want to disable auto rotation and if there
+	 * is another application that turned on the accelerators we still don't want
+	 * the input.
+	 */
+	if (appdata->accelerometers == FALSE)
+		return;
+
+	g_printerr("INFO: Setting to orientation: %s\n", orientation);
+
+	HildonPortraitFlags flags;
+
+
+	if (strcmp(orientation, "landscape") == 0) {
+		flags = HILDON_PORTRAIT_MODE_SUPPORT;
+		appdata->isportrait = FALSE;
+	} else if (strcmp(orientation, "portrait") == 0) {
+		flags = HILDON_PORTRAIT_MODE_REQUEST;
+		appdata->isportrait = TRUE;
+	} else {
+		g_printerr("ERROR: Orientation must be 'landscape' or 'portrait', not '%s'.\n", orientation);
+		return;
+	}
+
+	/* Switch the orientation of all open windows */
+	hildon_gtk_window_set_portrait_flags(GTK_WINDOW(appdata->window), flags);
+	
+	/* Tell the windows to redraw themselves, etc. */
+	rotate_scan_window();
+
+
+}
+
+
+
+gboolean device_is_portrait_mode()
+{
+    osso_rpc_t ret;
+    gboolean result = FALSE;
+
+    if (osso_rpc_run_system(appdata->osso, MCE_SERVICE, MCE_REQUEST_PATH,
+        MCE_REQUEST_IF, MCE_DEVICE_ORIENTATION_GET, &ret, DBUS_TYPE_INVALID) == OSSO_OK) {
+        g_printerr("INFO: DBus said orientation is: %s\n", ret.value.s);
+
+        if (strcmp(ret.value.s, MCE_ORIENTATION_PORTRAIT) == 0) {
+            result = TRUE;
+        }
+
+        osso_rpc_free_val(&ret);
+
+    } else {
+        g_printerr("ERROR: Call do DBus failed\n");
+    }
+
+    return result;
+}
+
+
+static DBusHandlerResult dbus_handle_mce_message(DBusConnection *con, DBusMessage *msg, gpointer data)
+{
+    DBusMessageIter iter;
+    const gchar *mode = NULL;
+    /* Could also catch other MCE messages here. */
+
+    if (dbus_message_is_signal(msg, MCE_SIGNAL_IF, MCE_DEVICE_ORIENTATION_SIG)) {
+        if (dbus_message_iter_init(msg, &iter)) {
+            dbus_message_iter_get_basic(&iter, &mode);
+            g_printerr("INFO: New orientation is now: %s\n", mode);
+            
+            set_orientation(mode);
+            
+        }
+    }
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+void orientation_enable_accelerometers()
+{
+	if (appdata->accelerometers == TRUE)
+		return;
+
+	/* TODO: Replace string "req_accelerometer_enable" with constant once we switched to final SDK */
+	if (osso_rpc_run_system(appdata->osso, MCE_SERVICE, MCE_REQUEST_PATH,
+			MCE_REQUEST_IF,	"req_accelerometer_enable", NULL, DBUS_TYPE_INVALID) == OSSO_OK) {
+		g_printerr("INFO: Accelerometers enabled\n");
+		appdata->accelerometers = TRUE;
+	} else {
+		g_printerr("WARN: Cannot enable accelerometers\n");
+	}
+}
+
+void orientation_disable_accelerometers()
+{
+	if (appdata->accelerometers == FALSE)
+		return;
+
+	/* TODO: Replace string "req_accelerometer_disable" with constant once we switched to final SDK */
+	if (osso_rpc_run_system(appdata->osso, MCE_SERVICE, MCE_REQUEST_PATH,
+			MCE_REQUEST_IF,	"req_accelerometer_disable", NULL, DBUS_TYPE_INVALID) == OSSO_OK) {
+		g_printerr("INFO: Accelerometers disabled\n");
+		appdata->accelerometers = FALSE;
+	} else {
+		g_printerr("WARN: Cannot disable accelerometers\n");
+	}
+}
+
+void orientation_init()
+{
+    printf("Entered orientation_init()\n");
+
+
+    printf("orientation_init(): about to run orientation_enable_accelerometers()\n");
+	/* Enable accelerometers */
+	orientation_enable_accelerometers();
+
+    printf("orientation_init(): about to run device_is_portrait_mode()\n");
+	/* Get current orientation from DBus and save into appdata */
+	appdata->isportrait = device_is_portrait_mode(appdata->osso);
+
+    printf("orientation_init(): about to dbus_bus_add_match()\n");
+	/* Add a matchin rule */
+	dbus_bus_add_match(appdata->con, MCE_MATCH_RULE, NULL);
+
+    printf("orientation_init(): about to dbus_connection_add_filter()\n");
+	/* Add the callback, which should be called, once the device is rotated */
+	dbus_connection_add_filter(appdata->con, dbus_handle_mce_message, NULL, NULL);
+}
+
+
 void initialize_gui(HildonProgram ** program, HildonWindow ** window, int *argc, char ***argv, gchar * app_name) {
     g_thread_init (NULL);
     gdk_threads_init ();
@@ -67,7 +209,9 @@ void initialize_gui(HildonProgram ** program, HildonWindow ** window, int *argc,
     g_set_application_name (app_name);
 
     /* Create the toplevel HildonWindow */
-    *window = HILDON_WINDOW (hildon_window_new ());
+    *window = HILDON_WINDOW (hildon_stackable_window_new ());
+
+    orientation_init();
 
     /* Connect destroying of the main window to gtk_main_quit */
     g_signal_connect (G_OBJECT (*window), "delete_event", G_CALLBACK (destroy_gui), NULL);
@@ -261,14 +405,19 @@ void search_dialog() {
 
 /* Create scan window contents */
 GtkWidget *create_scan_window() {
-    GtkWidget *vbox_left, *vbox_right, *hbox;
+    GtkWidget *table;
     GtkWidget *save_button;
     //PangoFontDescription *pfd;
 
     // create the packing structures
-    vbox_left = gtk_vbox_new (FALSE, 0);
-    vbox_right = gtk_vbox_new (FALSE, 0);
-    hbox = gtk_hbox_new (FALSE, 0);
+    appdata->scan_window_vbox_left = gtk_vbox_new (FALSE, 0);
+    appdata->scan_window_vbox_right = gtk_vbox_new (FALSE, 0);
+    
+    g_object_ref(appdata->scan_window_vbox_left);
+    g_object_ref(appdata->scan_window_vbox_right);
+
+    
+    appdata->scan_window_table = gtk_table_new (2, 2, FALSE);
 
     save_button = gtk_button_new_with_label("Save");
 
@@ -277,26 +426,38 @@ GtkWidget *create_scan_window() {
 
     // create the labels and other description stuff which will be on the right hand side of the screen
     strcpy(appdata->EAN_text, "0000000000000");
-    appdata->EANlabel = gtk_label_new (appdata->EAN_text);
+    //appdata->EANlabel = gtk_label_new (appdata->EAN_text);
+    appdata->EANlabel = hildon_entry_new(HILDON_SIZE_AUTO);
+    hildon_entry_set_text(GTK_ENTRY(appdata->EANlabel), appdata->EAN_text);
+    gtk_entry_set_editable(GTK_ENTRY(appdata->EANlabel), FALSE);
 
     strcpy(appdata->title_text, "unknown");
-    appdata->titlelabel = gtk_label_new (appdata->title_text);
-    gtk_label_set_line_wrap(appdata->titlelabel, TRUE); // set line wrapping so we can see all the output
+    //appdata->titlelabel = gtk_label_new (appdata->title_text);
+    //gtk_label_set_line_wrap(appdata->titlelabel, TRUE); // set line wrapping so we can see all the output
+    appdata->titlelabel = hildon_entry_new(HILDON_SIZE_AUTO);
+    hildon_entry_set_text(GTK_ENTRY (appdata->titlelabel), appdata->title_text);
+    gtk_entry_set_editable(GTK_ENTRY (appdata->titlelabel), FALSE);
 
     strcpy(appdata->author_text, "unknown");
-    appdata->authorlabel = gtk_label_new (appdata->author_text);
+    //appdata->authorlabel = gtk_label_new (appdata->author_text);
+    appdata->authorlabel = hildon_entry_new(HILDON_SIZE_AUTO);
+    hildon_entry_set_text(GTK_ENTRY (appdata->authorlabel), appdata->author_text);
+    gtk_entry_set_editable(GTK_ENTRY (appdata->authorlabel), FALSE);
 
     strcpy(appdata->price_text, "unknown");
-    appdata->pricelabel = gtk_label_new (appdata->price_text);
+    //appdata->pricelabel = gtk_label_new (appdata->price_text);
+    appdata->pricelabel = hildon_entry_new(HILDON_SIZE_AUTO);
+    hildon_entry_set_text(GTK_ENTRY (appdata->pricelabel), appdata->price_text);
+    gtk_entry_set_editable(GTK_ENTRY (appdata->pricelabel), FALSE);
 
     // pack the labels, etc
-    gtk_box_pack_start (GTK_BOX (vbox_right), appdata->EANlabel, TRUE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox_right), appdata->titlelabel, TRUE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox_right), appdata->authorlabel, TRUE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox_right), appdata->pricelabel, TRUE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (appdata->scan_window_vbox_right), appdata->EANlabel, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (appdata->scan_window_vbox_right), appdata->titlelabel, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (appdata->scan_window_vbox_right), appdata->authorlabel, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (appdata->scan_window_vbox_right), appdata->pricelabel, TRUE, TRUE, 0);
 
 // temporarily get rid of this until the sb stuff is working
-//    gtk_box_pack_start (GTK_BOX (vbox_right), save_button, TRUE, TRUE, 0);
+//    gtk_box_pack_start (GTK_BOX (appdata->scan_window_vbox_right), save_button, TRUE, TRUE, 0);
 
 
     // create the camera view window
@@ -314,23 +475,46 @@ GtkWidget *create_scan_window() {
     g_signal_connect (G_OBJECT (appdata->scan_button), "clicked", G_CALLBACK (scan_button_pressed), NULL);
 
     // start packing these objects
-    gtk_box_pack_start (GTK_BOX (vbox_left), appdata->screen, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox_left), appdata->scan_button, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (appdata->scan_window_vbox_left), appdata->screen, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (appdata->scan_window_vbox_left), appdata->scan_button, TRUE, TRUE, 0);
 
 
-    // now pack the left and right boxes into the overall horizontal box
-    gtk_box_pack_start (GTK_BOX (hbox), vbox_left, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (hbox), vbox_right, FALSE, FALSE, 0);
+    gtk_table_attach ((GtkTable *) appdata->scan_window_table, appdata->scan_window_vbox_left, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+    if(!appdata->isportrait)    
+        gtk_table_attach ((GtkTable *) appdata->scan_window_table, appdata->scan_window_vbox_right, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+    else
+        gtk_table_attach ((GtkTable *) appdata->scan_window_table, appdata->scan_window_vbox_right, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
-
-    return hbox;
+    return appdata->scan_window_table;
 }
 
 void update_scan_window() {
-    gtk_label_set_text ((GtkLabel *)appdata->EANlabel, appdata->EAN_text);
-    gtk_label_set_text((GtkLabel *)appdata->titlelabel, appdata->title_text);
-    gtk_label_set_text((GtkLabel *)appdata->authorlabel, appdata->author_text);
-    gtk_label_set_text((GtkLabel *)appdata->pricelabel, appdata->price_text);
+    //gtk_label_set_text ((GtkLabel *)appdata->EANlabel, appdata->EAN_text);
+    //gtk_label_set_text((GtkLabel *)appdata->titlelabel, appdata->title_text);
+    //gtk_label_set_text((GtkLabel *)appdata->authorlabel, appdata->author_text);
+    //gtk_label_set_text((GtkLabel *)appdata->pricelabel, appdata->price_text);
+    hildon_entry_set_text(GTK_ENTRY(appdata->EANlabel), appdata->EAN_text);
+    hildon_entry_set_text(GTK_ENTRY(appdata->titlelabel), appdata->title_text);
+    hildon_entry_set_text(GTK_ENTRY(appdata->authorlabel), appdata->author_text);
+    hildon_entry_set_text(GTK_ENTRY(appdata->pricelabel), appdata->price_text);
+}
+
+void rotate_scan_window()
+{
+    GtkWidget *box;
+    gtk_widget_hide_all(appdata->tab[0]);
+
+    gtk_container_remove(appdata->scan_window_table, appdata->scan_window_vbox_right);
+    if(!appdata->isportrait)    
+        gtk_table_attach ((GtkTable *) appdata->scan_window_table, appdata->scan_window_vbox_right, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+    else
+        gtk_table_attach ((GtkTable *) appdata->scan_window_table, appdata->scan_window_vbox_right, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+
+    // need to refresh the appdata->screen widget somehow...
+    
+    gtk_widget_show_all(appdata->scan_window_table);
+    gtk_widget_show_all(appdata->tab[0]);
+    //gdk_window_invalidate_rect(appdata->screen->window, NULL, FALSE);
 }
 
 
