@@ -1778,6 +1778,8 @@ int handle_barcode_data(BarcodeData *bcdata) //(char *data, int barcode_type)
 int parse_mixed_barcode_data(char *data)
 {
     char *urlp;
+    GtkWidget *banner;
+
     // the assumption here is that the barcode is definitely not an EAN
 
 
@@ -1805,6 +1807,8 @@ int parse_mixed_barcode_data(char *data)
     if (urlp){
         // we assume there's nothing of interest on the end
         printf("vCard!\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "vCard");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
 #ifdef N900        
         addVCard(urlp); // open the url
 #endif /* N900 */        
@@ -1815,29 +1819,6 @@ int parse_mixed_barcode_data(char *data)
 
 
 
-    // if it's QR code, check the iMode stuff
-    // http://www.nttdocomo.co.jp/english/service/imode/make/content/barcode/function/content/index.html
-    
-    if (strncmp(data, "CNTS:", 5)){
-        printf("i-mode content marker\n");
-        //type = data[10:14]
-        //lng = data[18:22]
-        //body = data[27:27+lng+1]
-        return TRUE;
-    }else if (strncmp(data, "MECARD:", 7)){ //#http://www.nttdocomo.co.jp/english/service/imode/make/content/barcode/function/application/common/index.html
-        printf("Phone book registration message\n");
-        return TRUE;
-    }else if (strncmp(data, "MATMSG:", 7)){
-        printf("Email sync message\n");
-        return TRUE;
-    }else if (strncmp(data, "MEBKM:", 6)){
-        printf("Bookmark registration message\n");
-        // we could try parsing this out and asking the user if they want to save it as a bookmark, or just open it in the browser
-        return TRUE;
-    }else if (strncmp(data,"LAPL:", 5)){
-        printf("i-appli sync message\n");
-        return TRUE;
-    }
 
 
 
@@ -1851,6 +1832,8 @@ int parse_mixed_barcode_data(char *data)
         urlp = urlp+4; // skip the URL: business
         
         printf("URL!\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "URL");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         // we assume there's nothing of interest on the end
         openURL(urlp); // open the url
         return TRUE;
@@ -1862,6 +1845,8 @@ int parse_mixed_barcode_data(char *data)
         urlp = urlp+6; // skip the URLTO: business
         
         printf("URL!\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "URL");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         // we assume there's nothing of interest on the end
         openURL(urlp); // open the url
         return TRUE;
@@ -1871,6 +1856,8 @@ int parse_mixed_barcode_data(char *data)
     urlp = strstr(data, "http://");
     if (urlp){        
         printf("URL!\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "URL");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         // we assume there's nothing of interest on the end
         openURL(urlp); // open the url
         return TRUE;
@@ -1878,16 +1865,58 @@ int parse_mixed_barcode_data(char *data)
     
     
     
+    
+    // if it's QR code, check the iMode stuff
+    // http://www.nttdocomo.co.jp/english/service/imode/make/content/barcode/function/content/index.html
+    
+    if (strncmp(data, "CNTS:", 5)){
+        printf("i-mode content marker\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "i-mode content marker");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        //type = data[10:14]
+        //lng = data[18:22]
+        //body = data[27:27+lng+1]
+        return TRUE;
+    }else if (strncmp(data, "MECARD:", 7)){ //#http://www.nttdocomo.co.jp/english/service/imode/make/content/barcode/function/application/common/index.html
+        printf("Phone book registration message\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "Phone book registration message");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        return TRUE;
+    }else if (strncmp(data, "MATMSG:", 7)){
+        printf("Email sync message\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "Email sync message");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        return TRUE;
+    }else if (strncmp(data, "MEBKM:", 6)){
+        printf("Bookmark registration message\n");
+        // we could try parsing this out and asking the user if they want to save it as a bookmark, or just open it in the browser
+        banner = hildon_banner_show_information (appdata->window, NULL, "Bookmark registration message");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        return TRUE;
+    }else if (strncmp(data,"LAPL:", 5)){
+        printf("i-appli sync message\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "i-appli sync message");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
+        return TRUE;
+    }
+    
+    
+    
+    
     // check for an email address...
     if (strncmp(data, "mailto:", 7)){        
         printf("Email address\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "Email address");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
     }
 
 
     // check for a telephone number...
     if (strncmp(data, "tel:", 4)){      
-        printf("Email address\n");
+        printf("Telephone number\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "Telephone number");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
     }
     
@@ -1896,6 +1925,8 @@ int parse_mixed_barcode_data(char *data)
     // check for a BIZCARD
     if (strncmp(data, "BIZCARD:", 8)){      
         printf("BIZCARD\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "BIZCARD");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
     }
 
@@ -1903,10 +1934,14 @@ int parse_mixed_barcode_data(char *data)
     // check for an SMS
     if (strncmp(data, "sms:", 4)){      
         printf("SMS\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "SMS");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
     }
     if (strncmp(data, "smsto:", 6)){
         printf("SMS\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "SMS");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
     }
 
@@ -1914,10 +1949,14 @@ int parse_mixed_barcode_data(char *data)
     // check for an MMS
     if (strncmp(data, "mms:", 4)){      
         printf("MMS\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "MMS");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
     }
     if (strncmp(data, "mmsto:", 6)){
         printf("MMS\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "MMS");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
     }
     
@@ -1925,6 +1964,8 @@ int parse_mixed_barcode_data(char *data)
     // check for geographic information
     if (strncmp(data, "geo:", 4)){      
         printf("Geographical information\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "Geographical information");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
     }
 
@@ -1933,6 +1974,8 @@ int parse_mixed_barcode_data(char *data)
     // Android specific - link to app in Market
     if (strncmp(data, "market:", 7)){      
         printf("Android Market link\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "Android Market link");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
     }
 
@@ -1941,6 +1984,8 @@ int parse_mixed_barcode_data(char *data)
     // iPhone rumoured - link to YouTube
     if (strncmp(data, "youtube:", 8)){      
         printf("Link to YouTube\n");
+        banner = hildon_banner_show_information (appdata->window, NULL, "Link to YouTube");
+        hildon_banner_set_timeout (HILDON_BANNER (banner), 3000);
         return TRUE;
     }
 
